using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace MaxDeRobot.Resources
{
    class FileController
    {
        private string[] fca;
        private int[] idpt;
        private string fname;
        private int rc;
        private StreamWriter sw;
        private StreamReader sr;
        private FileStream fs;

        public int[] indexpointer
        {
            get
            {
                return idpt;
            }
            set
            {
                idpt = value;
            }
        }
        
        public FileStream filestream
        {
            get
            {
                return fs;
            }
            set
            {
                fs = value;
            }
        }

        public StreamWriter streamwriter
        {
            get  //get accessor method
            {
                return sw;
            }
            set   //set accessor method
            {
                sw = value;
            }
        }

        public StreamReader streamreader
        {
            get
            {
                return sr;
            }
            set
            {
                sr = value;
            }
        }
        
        public int rowCount
        {
            get  //get accessor method
            {
                return rc;
            }
            set   //set accessor method
            {
                rc = value;
            }
        }

        public string fileName
        {
            get
            {
                return fname;
            }
            set
            {
                fname = value;
            }
        }

        public string[] fileContentArray
        {
            get
            {
                return fca;
            }
            set
            {
                fca = value;
            }
        }

        /*
         *Constructor 
         */
        public FileController(string fn)
        {
            fileName = fn;
            setFileBuffer();
        }

        
        /*Hier wordt een buffer gevuld met per index een regel van een max programma
         * pre: rowCount moet geset zijn met maxLines()
         * post: fileContentArray is geset
         */
        public void setFileBuffer()
        {
            maxLines();

            filestream = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.ReadWrite);
            fileContentArray = new string[rowCount];
            streamreader = new StreamReader(filestream);
            indexpointer = new int[rowCount];
            for (int i = 0; i < rowCount; i++ ) 
            {
                fileContentArray[i] = streamreader.ReadLine();
                indexpointer[i] = i;                
            }
            streamreader.Close();
            filestream.Close();
        }

        /* maximum aantal regels worden bepaald van het max programma
         * pre: fileName moet geset zijn (via constructor)
         * post: rowCount is geset
         */
        public void maxLines() 
        {
            try
            {
                filestream = new FileStream(fileName, FileMode.OpenOrCreate, FileAccess.ReadWrite);
                streamreader = new StreamReader(filestream);
                rc = 0;
                while (streamreader.ReadLine() != null) 
                    rc++;
                rowCount = rc;
                streamreader.Close();
                filestream.Close();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }           
        }

        public void AddLineToFile(string line)
        {
            fileContentArray[fileContentArray.Length] = line;
        }

        public string ReadFileLine(int pos)
        {
            string line = "";
            try
            {
                line = fileContentArray[pos];
            }
            catch (Exception exc)
            {
                line = "Nothing found on this position. " + Environment.NewLine + exc.Message;
            }

            return line;
        }

       //dit kan veel sneller 
        public string ReadWholeFile()
        {
            string text = "";
            try
            {
                for (int i = 0; i < fileContentArray.Length; i++)
                {
                    text = text + fileContentArray[i] + Environment.NewLine;
                }                
            }
            catch (Exception exc)
            {
                text = "Nothing found on this position. " + Environment.NewLine + exc.Message;
            }

            return text;
        }
        
        //dit moet nog wegschreven worden naar een bestand
        public void WriteFileLine(string text, int pos)
        {
            try
            {
                fileContentArray[pos] = text;
            }
            catch (Exception exc)
            {
                Console.WriteLine("Failed to add data on position " + pos +Environment.NewLine + exc.Message);
            }
        }

        public void WriteToTextFile()
        {
//            FileStream f = new FileStream(fname, FileMode.OpenOrCreate, FileAccess.ReadWrite);
//            StreamWriter sw = new StreamWriter(f);
            try
            {
                String temp = "";
                for(int i =  0; i < fileContentArray.Length; i++)
                {
//                    sw.WriteLine(fileContentArray[i]);
                    temp = temp + fileContentArray[i] + "\n";
                }
                sw.Write(temp);
                sw.Close();
            }
            catch (Exception exc)
            {
                Console.WriteLine("File could not be opened or written to." + Environment.NewLine + "Please verify that the filename is correct, and that you have read permissions for the desired directory." + Environment.NewLine + Environment.NewLine + "Exception: " + exc.Message);
            }
            finally
            {
                // Close the object if it has been created.
                if (sw != null)
                {
                    sw.Close();
                }
            }
        }
   }
}
